// * Copyright (C) 2002-2004 Sergei Chekanov (ANL) chekanov@mail.desy.de 
// * This file is part of the RunMC project
// * 
// * Permission is hereby granted, free of charge, to any person obtaining a copy
// * of this software and associated documentation files (the "Software"), to
// * deal in the Software without restriction, including without limitation the
// * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// * sell copies of the Software, and to permit persons to whom the Software is
// * furnished to do so, subject to the following conditions:
// *
// * The above copyright notice and this permission notice shall be included in
// * all copies or substantial portions of the Software.
// * 
// * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// * SERGEI  CHEKANOV BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
// * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// * SOFTWARE.
// * 
// * Except as contained in this notice, the name of Sergei Chekanov shall
// * not be used in advertising or otherwise to promote the sale, use or other
// * dealings in this Software without prior written authorization from
// * Sergei Chekanov.
// * 
#include <iostream>
#include <fstream>
#include <sys/stat.h>

#include <SetIni.h> // include class containing initializations for MC 
#include <SetHisto.h> // initialization of histograms

#include <stdio.h>
#include <stdlib.h>
#include <WSCstring.h>
#include <WSCvariant.h>
extern WSCstring McRunFileName;

// to decomment input file
extern int decom_inverse( char * file_in_name, char * file_out_name,  char comment[] ); 

using namespace std;

 
//----------------------------------------------------------
//Function for the event procedure
//----------------------------------------------------------
int MCread (char * fname )  {
  //do something...
    extern SetIni inival;
    extern SetHisto inihis;

   char * ReadINT="integer.tmp";    
   char * ReadREA="real.tmp";
   char * ReadSTR="string.tmp";
   char * ReadHISTO="histo.tmp"; 
   char * ReadOPTIONS="options.tmp"; 

 
// set to zero
   inival.Nevents=0;
   char tt[1]="";
   for (int i=0; i<=500; i++) {
       inihis.HistoType[i]=0; inihis.HistoMax[i]=0.; inihis.HistoMin[i]=0.; inihis.HistoBins[i]=0;
       inihis.Histo2D[i]=1;
       inihis.HistoWeight[i]=1.;
       inihis.HistoName[i] = new char[strlen(tt)+1]; // empty
       strcpy(inihis.HistoName[i],tt);
       inihis.HistoComm[i] = new char[strlen(tt)+1]; // empty
       strcpy(inihis.HistoComm[i],tt);
      };
 

// decomment card file
   int iicheck=0;
   iicheck=decom_inverse(fname,ReadINT,"I");
   if (iicheck !=  0 ){ cout << "Cannot find file " << ReadINT << endl; return 1; }
   iicheck=decom_inverse(fname,ReadREA,"R");
   if (iicheck !=  0 ){ cout << "Cannot find file " << ReadREA << endl; return 1; }    
   iicheck=decom_inverse(fname,ReadSTR,"S");
   if (iicheck !=  0 ){ cout << "Cannot find file " << ReadSTR << endl; return 1; }
   iicheck=decom_inverse(fname,ReadHISTO,"H");
   if (iicheck !=  0 ){ cout << "Cannot find file " << ReadHISTO << endl; return 1; }
   iicheck=decom_inverse(fname,ReadOPTIONS,"O");
   if (iicheck !=  0 ){ cout << "Cannot find file " << ReadOPTIONS << endl; return 1; }



// read files 
   char   buff[60];
   float  VRea;    
   char   VSOpt[20];

// loop over all real  
   int nreal=0; char * tmp; 
   FILE *ls2 = fopen(ReadREA,"r");
   if (ls2  == NULL ){ cout << "Cannot find " <<  ReadREA << endl; return 1; }
   while ( !feof(ls2) ) { 
     fscanf(ls2,"%s %f \n", buff, &VRea);  
     tmp = new char[strlen(buff)+1]; strcpy(tmp,buff);
     if (strcmp(tmp,"R-ENERGY1") == 0 )  inival.Energy1=VRea;
     if (strcmp(tmp,"R-ENERGY2") == 0 )  inival.Energy2=VRea;
     if (strcmp(tmp,"R-YCUT") == 0 )     inival.JetYcut=VRea;
     if (strcmp(tmp,"R-Q2MIN") == 0 )    inival.Q2min=VRea;
     if (strcmp(tmp,"R-Q2MAX") == 0 )    inival.Q2max=VRea;
     if (strcmp(tmp,"R-XMIN") == 0 )     inival.Xmin=VRea;
     if (strcmp(tmp,"R-XMAX") == 0 )     inival.Xmax=VRea;
     if (strcmp(tmp,"R-YMIN") == 0 )     inival.Ymin=VRea;
     if (strcmp(tmp,"R-YMAX") == 0 )     inival.Ymax=VRea;
     if (strcmp(tmp,"R-WMIN") == 0 )     inival.Wmin=VRea;
     if (strcmp(tmp,"R-WMAX") == 0 )     inival.Wmax=VRea;
     if (strcmp(tmp,"R-ETMIN") == 0 )    inival.ETmin=VRea;
     if (strcmp(tmp,"R-ETMAX") == 0 )    inival.ETmax=VRea;
     if (strcmp(tmp,"R-ETAMIN") == 0 )   inival.ETAmin=VRea;
     if (strcmp(tmp,"R-ETAMAX") == 0 )   inival.ETAmax=VRea;
     if (strcmp(tmp,"R-NTlumi") == 0 )   inival.NTlumi=VRea;
//     cout << tmp << " " << VRea << endl; 
     nreal++;
   }
   fclose(ls2);



// loop over all integer 
   int   VInt;
   int inter=0;
   FILE *ls3 = fopen(ReadINT,"r");
   if (ls3  == NULL ){ cout << "Cannot find " <<  ReadINT << endl; return 1; }
   while ( !feof(ls3) ) {
     fscanf(ls3,"%s %d \n", buff, &VInt);
     tmp = new char[strlen(buff)+1]; strcpy(tmp,buff);
     if (strcmp(tmp,"I-EVENTS") == 0 )  inival.Nevents=VInt;
     if (strcmp(tmp,"I-MCTYPE") == 0 )  inival.MCtype=VInt;
     if (strcmp(tmp,"I-PROCESS") == 0 ) inival.MCprocess=VInt;
     if (strcmp(tmp,"I-ID1") == 0 )     inival.IDparticle1=VInt;
     if (strcmp(tmp,"I-ID2") == 0 )     inival.IDparticle2=VInt;
     if (strcmp(tmp,"I-PDF1") == 0 )    inival.PDFparticle1=VInt;
     if (strcmp(tmp,"I-PDF2") == 0 )    inival.PDFparticle2=VInt;
     if (strcmp(tmp,"I-NTTYPE") == 0 )  inival.NTwhat=VInt;
     if (strcmp(tmp,"I-NTREAD") == 0 )  inival.NTread=VInt;
     if (strcmp(tmp,"I-FSTYPE") == 0 )  inival.FStype=VInt;
     if (strcmp(tmp,"I-JETTYPE") == 0 ) inival.JetType=VInt;
     if (strcmp(tmp,"I-NJETS") == 0 )   inival.JetNfixed=VInt;
     if (strcmp(tmp,"I-ANGLE") == 0 )   inival.JetAngle=VInt;
     if (strcmp(tmp,"I-RECOMB") == 0 )  inival.JetRecomb=VInt;
     if (strcmp(tmp,"I-STOP") == 0 )    inival.Stop=VInt;
     if (strcmp(tmp,"I-RENICE") == 0 )  inival.Renice=VInt;
     if (strcmp(tmp,"I-STEER") == 0 )   inival.Steering=VInt;
//   cout << tmp  << " " << VInt << endl;
     inter++;
   }
   fclose(ls3);

// loop over all strings  
   int nstring=0;
   char VCha[300];
   FILE *ls4 = fopen(ReadSTR,"r");
   if (ls4  == NULL ){ cout << "Cannot find " <<  ReadSTR << endl; return 1; }
   while ( !feof(ls4) ) {
     fscanf(ls4,"%s %s \n", buff, VCha);
// next strings
     tmp = new char[strlen(buff)+1]; strcpy(tmp,buff);
     if (strcmp(tmp,"S-PROJECT") == 0 ) {inival.MOutput = new char[strlen(VCha)+1]; strcpy(inival.MOutput,VCha);};
     if (strcmp(tmp,"S-NTNAME") == 0 ) {inival.NTname = new char[strlen(VCha)+1]; strcpy(inival.NTname,VCha);};
     if (strcmp(tmp,"S-NTDIR") == 0 ) {inival.NTdir = new char[strlen(VCha)+1]; strcpy(inival.NTdir,VCha);};
     if (strcmp(tmp,"S-PIPE") == 0 ) {inival.PipeName = new char[strlen(VCha)+1]; strcpy(inival.PipeName,VCha);};
//   cout << tmp << " " << VCha << endl;
     nstring++;
   }
   fclose(ls4);


// read option files 
   FILE *ls5 = fopen(ReadOPTIONS,"r");
   if (ls5  == NULL ){ cout << "Cannot find " <<  ReadOPTIONS  << endl; return 1; }
   fscanf(ls5,"%s", VSOpt);
   fscanf(ls5,"%d %d %d", &inihis.HistoOptions[0],&inihis.HistoOptions[1],&inihis.HistoOptions[2]);
   fscanf(ls5,"%d %d %d", &inihis.HistoOptions[3],&inihis.HistoOptions[4],&inihis.HistoOptions[5]); //canvas
   fscanf(ls5,"%d %d %d", &inihis.HistoOptions[6],&inihis.HistoOptions[7],&inihis.HistoOptions[8]);
   fscanf(ls5,"%d %d %d", &inihis.HistoOptions[9],&inihis.HistoOptions[10],&inihis.HistoOptions[11]);
   fscanf(ls5,"%d %d %d", &inihis.HistoOptions[12],&inihis.HistoOptions[13],&inihis.HistoOptions[14]); // not used
   fscanf(ls5,"%d %d %d", &inihis.HistoOptions[15],&inihis.HistoOptions[16],&inihis.HistoOptions[17]);
   fscanf(ls5,"%d %d %d \n", &inihis.HistoOptions[18],&inihis.HistoOptions[19]);
   fclose(ls5);

// now get histograms 
     int i=1;
     char test0[60]; 
     char test1[60];
     char test2[60];
     char t0[60];
     char t1[60];
     char t2[60];
     char bb[81];
     inihis.HistoN=1;
     WSCvariant  s1,s2,s3;
     struct stat buf;
     int isi = stat (ReadHISTO,&buf);
     int file_size=0; file_size=buf.st_size;

     if (file_size>0 && isi == 0 ) { 
     FILE *ls6 = fopen(ReadHISTO,"r");
     while ( !feof(ls6) ) { 
     fscanf(ls6,"%s %s", test0, test1); fgets(bb,70,ls6);
     inihis.HistoType[i]=i; 
     inihis.HistoName[i] = new char[strlen(test1)];
     strcpy(inihis.HistoName[i],test1);
// get comment
     WSCstring tmp(bb); 
     tmp.delHeadSpace();
     tmp.delTailSpace();
     inihis.HistoComm[i] = new char[strlen((char*)tmp)];
     strcpy(inihis.HistoComm[i],(char*)tmp);
     fscanf(ls6,"%s %d %s %s %d %s \n",
     test2,&inihis.Histo2D[i],t0,t1,&inihis.HistoBins[i],t2);
     s1=t0; 
     s2=t1; 
     s3=t2; 
     inihis.HistoMin[i]=s1.getDouble();
     inihis.HistoMax[i]=s2.getDouble(); 
     inihis.HistoWeight[i]=s3.getDouble(); 
     s1.clear();
     s2.clear();
     s3.clear();
//    cout << inihis.HistoType[i] << inihis.HistoName[i] << endl; 
//    cout << inihis.HistoName[i]   <<  inihis.HistoMin[i] << inihis.HistoMax[i] << inihis.HistoBins[i] << endl;
     i++;
     };
     fclose(ls6);
     };

// set to zero, if no histograms 
     inihis.HistoN=i;
    

// remove temporary file
    remove (ReadINT);
    remove (ReadREA); 
    remove (ReadSTR); 
    remove (ReadHISTO); 
    remove (ReadOPTIONS);

   return 0;

}
