/*
 * GeneralOptionPane.java - General settings
 * :tabSize=8:indentSize=8:noTabs=false:
 * :folding=explicit:collapseFolds=1:
 *
 * Copyright (C) 1999, 2004 Slava Pestov
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package runmcconsole.options;

//{{{ Imports
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.util.Vector;
import org.gjt.sp.jedit.gui.*;
import org.gjt.sp.jedit.*;
//}}}

public class GeneralOptionPane extends AbstractOptionPane
{
	//{{{ GeneralOptionPane constructor
	public GeneralOptionPane()
	{
		super("runmcconsole.general");
	} //}}}

	//{{{ Protected members

	//{{{ _init() method
	protected void _init()
	{
		addComponent(commandoToolBar = new JCheckBox(jEdit.getProperty(
			"options.runmcconsole.general.commando.toolbar")));
		commandoToolBar.getModel().setSelected(jEdit.getBooleanProperty(
			"commando.toolbar.enabled"));

		font = new FontSelector(jEdit.getFontProperty("runmcconsole.font"));
		addComponent(jEdit.getProperty("options.runmcconsole.general.font"),font);

		addComponent(jEdit.getProperty("options.runmcconsole.general.bgColor"),
			bgColor = createColorButton("runmcconsole.bgColor"));
		addComponent(jEdit.getProperty("options.runmcconsole.general.plainColor"),
			plainColor = createColorButton("runmcconsole.plainColor"));
		addComponent(jEdit.getProperty("options.runmcconsole.general.infoColor"),
			infoColor = createColorButton("runmcconsole.infoColor"));
		addComponent(jEdit.getProperty("options.runmcconsole.general.warningColor"),
			warningColor = createColorButton("runmcconsole.warningColor"));
		addComponent(jEdit.getProperty("options.runmcconsole.general.errorColor"),
			errorColor = createColorButton("runmcconsole.errorColor"));
	} //}}}

	//{{{ _save() method
	protected void _save()
	{
		jEdit.setBooleanProperty("commando.toolbar.enabled",
			commandoToolBar.getModel().isSelected());

		jEdit.setFontProperty("runmcconsole.font",font.getFont());

		jEdit.setColorProperty("runmcconsole.bgColor",
			bgColor.getBackground());
		jEdit.setColorProperty("runmcconsole.plainColor",
			plainColor.getBackground());
		jEdit.setColorProperty("runmcconsole.infoColor",
			infoColor.getBackground());
		jEdit.setColorProperty("runmcconsole.warningColor",
			warningColor.getBackground());
		jEdit.setColorProperty("runmcconsole.errorColor",
			errorColor.getBackground());
	} //}}}

	//}}}

	//{{{ Private members

	//{{{ Instance variables
	private JCheckBox commandoToolBar;
	private FontSelector font;
	private JButton bgColor;
	private JButton plainColor;
	private JButton infoColor;
	private JButton warningColor;
	private JButton errorColor;
	//}}}

	//{{{ createColorButton() method
	private JButton createColorButton(String property)
	{
		final JButton b = new JButton(" ");
		b.setBackground(jEdit.getColorProperty(property));
		b.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent evt)
			{
				Color c = JColorChooser.showDialog(
					GeneralOptionPane.this,
					jEdit.getProperty("colorChooser.title"),
					b.getBackground());
				if(c != null)
					b.setBackground(c);
			}
		});

		b.setRequestFocusEnabled(false);
		return b;
	} //}}}

	//}}}
}
