// * This code is licensed under:
// * JRunMC License, Version 1.0
// * - for license details see http://runmc.sourceforge.net/License.txt
// *
// * Copyright (c) 2005 by S.Chekanov (chekanov@mail.desy.de). 
// * All rights reserved.
package JRunMC;


import java.awt.Component;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.datatransfer.DataFlavor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.EOFException;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.Comparator;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeSet;
import java.lang.*;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.JTextComponent;

/**
 *  Static singleton class organizing actions, mainly depending on the command 
 *  menus, and action related assisting routines. This class manages the system's
 *  idle time counter. 
 * <p>
 */
public class ActionHandler
{
   private static ActionListener actionListener = new MainActionListener();
   private static DocumentListener docListener = new DocumentChangeListener();
   private static long lastUseTime;
   private static Action clearClipAction;
   
   /**
    *  Singleton class. 
    */
   private ActionHandler ()
   {
   }

   public static void init (  )
   {
      lastUseTime = System.currentTimeMillis();
      
      // define Action "menu.edit.clearclip"
      clearClipAction = new AbstractAction( 
            ResourceLoader.getCommand( "menu.edit.clearclip" ),
            ResourceLoader.getImageIcon( "clearclip" ) )
      {
         public void actionPerformed( ActionEvent arg )
         {
            actionListener.actionPerformed( arg );
         }
      };
      clearClipAction.putValue( Action.SHORT_DESCRIPTION, 
            ResourceLoader.getCommand( "toolbar.clearclip.tooltip" ) );
      clearClipAction.putValue( Action.ACTION_COMMAND_KEY, "menu.edit.clearclip" );
      
   }  // init
   
   /** Returns the "ClearClipboard" action which is defined in this class. */
   public static Action getClearClipAction ()
   {
      return clearClipAction;
   }
   
   /** Returns the <code>ActionListener</code> which is defined in this class. */
   public static ActionListener getMainActionListener ()
   {
      return  actionListener;
   }

   /** Returns the actual idle time of the application. */ 
   public static long getIdleTime ()
   {
      return System.currentTimeMillis() - lastUseTime;
   }

   /** Resets the application's idle counter to zero (current time). */
   public static void resetIdleTime ()
   {
      lastUseTime = System.currentTimeMillis();
   }
   
   /** Causes a new password file to be created and displayed in the GUI.
    *  (Includes user passphrase input.)
    */



/**
 *  An instance of this class listenes to <code>JTextComponent</code>s and 
 *  resets the system's idle-time counter whenever a modification occurs on
 *  a component's text.      
 */
private static class DocumentChangeListener implements DocumentListener
{

public void changedUpdate ( DocumentEvent e )
{}

public void insertUpdate ( DocumentEvent e )
{
   resetIdleTime();
}

public void removeUpdate ( DocumentEvent e )
{
   resetIdleTime();
}
} // inner class DocumentChangeListener
 
/**
 *  This class is the main listener to action commands. Those are majorly
 *  coming from the GUI's menubars and toolbars.
 *  
 */
private static class MainActionListener implements ActionListener
{

public void actionPerformed ( ActionEvent arg0 )
{

   String cmd, hstr;
   boolean isSelected;
   
   lastUseTime = System.currentTimeMillis();
   cmd = arg0.getActionCommand();

//   System.out.println("Debug: action:"+cmd);  
 
   if ( cmd.equals( "" ) )
      return;
  
 
   else if ( cmd.equals( "menu.help.mc" ) )
   {

    try {  
    Global.mainFrame.ShowMess("Starting Acroread to open Monte Carlo manual.. wait");  
    Process p = Runtime.getRuntime().exec(
    SetEnv.RUNDIR+"/main/runmc/show_manual_mc.sh "+Integer.toString(SetIni.MCtype)+" &");
    } catch (Exception err) {  err.printStackTrace(); }

   }
  else if ( cmd.equals( "menu.help.runmc" ) )
   {

    try {
     Global.mainFrame.ShowMess("Starting Acroread to open RunMC manual.. wait");
     Process p = Runtime.getRuntime().exec(SetEnv.RUNDIR+"/main/runmc/show_manual.sh &");
        } catch (Exception err) {  err.printStackTrace(); }


 }
 else if ( cmd.equals( "menu.help.paper" ) )
   {
       try {
        Global.mainFrame.ShowMess("Starting Acroread to open the RunMC paper.. wait");
        Process p = Runtime.getRuntime().exec(
        SetEnv.RUNDIR+"/main/runmc/show_paper.sh &");
        } catch (Exception err) {  err.printStackTrace();
        } 

   }

else if ( cmd.equals( "menu.file.exit" ) )
   {

    JOptionPane dialog1 = new JOptionPane();
    int ask=dialog1.showConfirmDialog(dialog1, "Exit JRunMC?",
    "Question", JOptionPane.YES_NO_OPTION,
    JOptionPane.QUESTION_MESSAGE, null );
    if (ask == JOptionPane.YES_OPTION) {

          String sh = "bash"; String cop = "-c";
          String cmd1 = "pkill histoview";
          String[] exe1 = new String[] {sh, cop, cmd1};
          try { Process p = Runtime.getRuntime().exec(exe1);
         //       System.out.println("histoview killed");
          } catch (Exception err) {err.printStackTrace(); }


          cmd1 = "pkill ntupview";
          exe1 = new String[] {sh, cop, cmd1};
          try { Process p = Runtime.getRuntime().exec(exe1);
          //      System.out.println("ntupview  killed");
          } catch (Exception err) {err.printStackTrace(); }


// save options
      if (!Options.checkOption()) {

      JOptionPane dialog2 = new JOptionPane();
      String s="Some options are different from the defaut values. Save them for future?";
      String mesg = "<html><b>"+s+"</b></html>";

       int ask2=dialog2.showConfirmDialog(dialog1, mesg, "Question",
              JOptionPane.YES_NO_OPTION,JOptionPane.QUESTION_MESSAGE, null );
       if (ask2 == JOptionPane.YES_OPTION) Options.save();
       if (ask2 == JOptionPane.NO_OPTION)  Options.reset();
       }

     System.exit(0);
    } 

 }
   else if ( cmd.equals( "menu.help.about" ) )
   {
        Global.mainFrame.ShowMess("About dialog");
        new AboutDialog();
   }
 

  else if ( cmd.equals( "menu.help.system" ) )
   {
        Global.mainFrame.ShowMess("System information");
        new SystemInfo();
   }


 else if ( cmd.equals( "menu.project.browser" ) )
   {
          String sh = "bash"; String cop = "-c";
          String cmd1 = "$RUNMC/main/runmc/show_jedit.sh";
          String[] exe1 = new String[] {sh, cop, cmd1};
          try { Process p = Runtime.getRuntime().exec(exe1);
               Global.mainFrame.ShowMess("Start project browser.."); 
          } catch (Exception err) {err.printStackTrace(); }
          //System.out.println("start browser");

     }  

else if ( cmd.equals( "menu.project.save" ) )
   {

    JOptionPane dialogsaveRMC = new JOptionPane();
    int asksaveRMC=dialogsaveRMC.showConfirmDialog(dialogsaveRMC,
     "Save RMC project?",
     "Question", JOptionPane.YES_NO_OPTION,
       JOptionPane.QUESTION_MESSAGE, null );
       if (asksaveRMC == JOptionPane.YES_OPTION) {
        // write file
        String sn=SetIni.MOutput+".mc";
        try {SetIni.FileWriteMC(sn);
           Global.mainFrame.ShowMess(sn+" file was saved");
        } catch (Exception e) {
           Global.mainFrame.ShowMess(SetIni.MOutput+".mc  cannot be saved!");

        }
         // check file
         File oldrmc= new File(SetEnv.RUNDIR+"/archive/"+SetIni.MOutput+".rmc");
         String cmd2 = SetEnv.RUNDIR+"/main/runmc/proj_create.sh &";

         if (oldrmc.exists()) {
         JOptionPane dialog3 = new JOptionPane();
         int ask3=dialog3.showConfirmDialog(dialog3,
         "RMC file with this name exists, overwrite?",
         "Question", JOptionPane.YES_NO_OPTION,
         JOptionPane.QUESTION_MESSAGE, null );
         if (ask3 == JOptionPane.YES_OPTION) {

         try {
           Process p = Runtime.getRuntime().exec(cmd2);
           Global.mainFrame.ShowMess(SetIni.MOutput+".rmc  project was saved");
         } catch (Exception err) {  err.printStackTrace();}
         } else if (ask3 == JOptionPane.NO_OPTION) {
            Global.mainFrame.ShowMess(SetIni.MOutput+".rmc  was canceled"); 
         }
         } else { // if file does not exits
         try {Process p = Runtime.getRuntime().exec(cmd2);
         } catch (Exception err) {  err.printStackTrace();}
          Global.mainFrame.ShowMess( SetIni.MOutput+".rmc  project was saved" );
         }
            } else if (asksaveRMC == JOptionPane.NO_OPTION) {
        } else { System.out.print("Something is wrong"); };


   }

else if ( cmd.equals( "menu.file.savemc" ) )
   {

      JOptionPane dialog1 = new JOptionPane();
      int ask=dialog1.showConfirmDialog(dialog1, "Save current settings?",
       "Question", JOptionPane.YES_NO_OPTION,
       JOptionPane.QUESTION_MESSAGE, null );
       if (ask == JOptionPane.YES_OPTION) {
                 // write file
                 String sn=SetIni.MOutput+".mc";
                 // save histograms first
                 try {
                   SetIni.FileWriteMC(sn);
               } catch (Exception e) {
                  System.out.println("Error in file openning");
                  // show error
                  JOptionPane dialog2 = new JOptionPane();
                  dialog2.showMessageDialog(dialog2, "Cannot write "+sn,
                  "Error", JOptionPane.ERROR_MESSAGE );  };
                     Global.mainFrame.ShowMess( sn+" file was saved"); 
                    } else if (ask == JOptionPane.NO_OPTION) {
                    } else {
                      System.out.print("Something is wrong");
                    }

}

else if ( cmd.equals( "menu.project.info" ) )
   {

// start RMC viewer
        try {
         Global.mainFrame.ShowMess("Start RMC viewer ..");
         RMCviewer rmcv = new RMCviewer();
        } catch (Exception err) {  err.printStackTrace();
        }    

}

  else
      System.out.println("** unrecognized command: " + cmd );
   
}  // actionPerformed


}






}
