/*
 *  HtmlBrowserDialog in org.jpws.front.util
 *  file: HtmlBrowserDialog.java
 * 
 *  Project Jpws-Front
 *  @author Wolfgang Keller
 *  Created 14.07.2005
 *  Version
 * 
 *  Copyright (c) 2005 by Wolfgang Keller, Munich, Germany
 * 
 This program is not freeware software but copyright protected to the author(s)
 stated above. However, you can use, redistribute and/or modify it under the terms 
 of the GNU General Public License as published by the Free Software Foundation, 
 version 2 of the License.

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 http://www.gnu.org/copyleft/gpl.html.
 */

package JRunMC;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.HeadlessException;
import java.io.IOException;
import java.net.URL;

import javax.swing.BorderFactory;
import javax.swing.JDialog;
import javax.swing.JScrollPane;


/**
 *  Dialog to display and browse a HTML text document.  
 */
public class HtmlBrowserDialog extends JDialog
{
   private HtmlDialogPane htmlPane;
   private DialogButtonBar buttonBar;

/**
 * Creates a non-modal dialog.
 * @throws java.awt.HeadlessException
 */
public HtmlBrowserDialog () throws HeadlessException
{
   super();
   init();
}

/**
 * @param owner
 * @param modal
 * @throws java.awt.HeadlessException
 */
public HtmlBrowserDialog ( Dialog owner, boolean modal )
      throws HeadlessException
{
   super( owner, modal );
   init();
}

/**
 * @param owner
 * @param modal
 * @throws java.awt.HeadlessException
 */
public HtmlBrowserDialog ( Frame owner, boolean modal )
      throws HeadlessException
{
   super( owner, modal );
   init();
}

/**
 * @param owner
 * @param title
 * @param modal
 * @throws java.awt.HeadlessException
 */
public HtmlBrowserDialog ( Dialog owner, String title, boolean modal )
      throws HeadlessException
{
   super( owner, title, modal );
   init();
}

/**
 * @param owner
 * @param title
 * @param modal
 * @throws java.awt.HeadlessException
 */
public HtmlBrowserDialog ( Frame owner, String title, boolean modal )
      throws HeadlessException
{
   super( owner, title, modal );
   init();
}

public void init ()
{
   htmlPane = new HtmlDialogPane();
//   htmlPane.addHyperlinkListener( Global.createHyperLinkListener() );
   htmlPane.setPreferredSize( new Dimension(500,400) );
   htmlPane.getScrollPane().setHorizontalScrollBarPolicy( 
         JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );
   
   buttonBar = new DialogButtonBar( DialogButtonBar.OK_BUTTON );
   
   getContentPane().add( htmlPane, BorderLayout.CENTER );
   getContentPane().add( buttonBar, BorderLayout.SOUTH );
   
   pack();
}

public void show ()
{
   Util.centreWithin( this.getOwner(), this );
   super.show();
}

public void setText ( String text )
{
   htmlPane.getEditorPane().setText( text );
}

public void setPage ( URL page ) throws IOException
{
   htmlPane.getEditorPane().setPage( page );
}



}
