
package JRunMC;

import javax.swing.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

public class ObjectChooser extends JDialog
                           implements MouseListener,
                                      ActionListener {

  SetupTheEG owner;
  String currentDir;
  String selectedObject;
  boolean locked;
  boolean done;
  JList dirList = new JList();
  JList objList = new JList();
  JTextArea infoText= new JTextArea();
  JComboBox dirsel = new JComboBox();
  JButton cancelButton = new JButton("Cancel");
  JButton okButton = new JButton("Select");

  public ObjectChooser(SetupTheEG frame, String dir) {
    super(frame, "Choose Object", true);
    owner = frame;
    currentDir = dir;

    getContentPane().setLayout(new BorderLayout());

    JPanel panel = new JPanel(new GridLayout(1,3));

    dirList.addMouseListener(this);
    panel.add(getTitlePanel("Directories", new JScrollPane(dirList)));
    objList.addMouseListener(this);
    panel.add(getTitlePanel("Objects", new JScrollPane(objList)));
    infoText.setEditable(false);
    infoText.setLineWrap(true);
    infoText.setWrapStyleWord(true);
    panel.add(getTitlePanel("Info", new JScrollPane(infoText)));

    getContentPane().add(panel, BorderLayout.CENTER);

    dirsel.addActionListener(this);
    getContentPane().add(dirsel, BorderLayout.NORTH);

    panel = new JPanel();
    cancelButton.addActionListener(this);
    okButton.addActionListener(this);
    panel.add(cancelButton);
    panel.add(okButton);
    getContentPane().add(panel, BorderLayout.SOUTH);
      

    setSize(400,300);
    setup(dir);

    setVisible(true);

  }

  public static JPanel getTitlePanel(String title, JComponent obj) {
    JPanel panel = new JPanel(new BorderLayout());
    panel.add(new JLabel(title), BorderLayout.NORTH);
    panel.add(obj, BorderLayout.CENTER);
    return panel;
  }

  public String selected() {
    if ( selectedObject == null || !done ) return null;
    return currentDir + selectedObject;
  }

  private void setInfo() {
    if ( selectedObject == null ) {
      infoText.setText("");
      return;
    }
    LinkedList ret = owner.exec("fulldescribe " + currentDir + selectedObject);
    String s = "-- Name --\n" + selectedObject;
    if ( ret.size() > 0 ) ret.remove(0);
    if ( ret.size() > 0 ) s += "\n-- Class --\n" + (String)ret.remove(0);
    s += "\n-- Description --\n";
    String line = "";
    while ( ret.size() > 0 &&
	    !(line = (String)ret.remove(0)).equals("Interfaces:") )
      s += line;
    infoText.setText(s);
    repaint();

  }


  private void setup(String dir) {
    if ( locked ) return;
    if ( dir == null ) return;
    locked = true;

    currentDir = dir;

    LinkedList ret = owner.exec("ls " + currentDir);
    Vector files = new Vector();
    Vector dirs = new Vector();
    dirs.add("..");

    while ( ret.size() > 0 ) {
      String s = (String)ret.remove(0);
      s = s.substring(currentDir.length());
      if ( s.charAt(s.length() - 1) == '/' )
	dirs.add(s);
      else
	files.add(s);
    }
    dirList.setListData(dirs);
    objList.setListData(files);

    dirsel.removeAllItems();
    for ( int i = 0; i < currentDir.length(); ++i ) {
      if ( currentDir.charAt(i) == '/' )
	dirsel.addItem(currentDir.substring(0, i + 1));
    }
    dirsel.setSelectedIndex(dirsel.getItemCount() - 1);
    selectedObject = null;
    setInfo();

    repaint();

    locked = false;

  }

  public void actionPerformed(ActionEvent e) {
    if ( e.getSource() == cancelButton ) {
      selectedObject = null;
      dispose();
    }
    if ( e.getSource() == okButton ) {
      if ( selectedObject != null ) {
	done = true;
	dispose();
      } else if ( dirList.getSelectedValue() != null ) {
	String sel = (String)dirList.getSelectedValue();
	if ( sel == ".." )
	  updir();
	else
	  setup(currentDir + sel);
      }
    }
    if ( !locked && e.getSource() == dirsel ) {
      if ( dirsel.getSelectedItem() != null )
	setup((String)dirsel.getSelectedItem());
    }
  }

  public void updir() {
    if ( currentDir.equals("/") ) return;
    int pos = currentDir.lastIndexOf('/', currentDir.length() - 2);
    setup(currentDir.substring(0, pos + 1));
  }

  public void mouseClicked(MouseEvent e) {
    if ( e.isMetaDown() || e.isControlDown() ||
	 e.isShiftDown() || e.isAltDown() || e.isAltGraphDown() ) return;
    if ( e.getSource() == dirList && e.getClickCount() >= 2 ) {
      String sel = (String)dirList.getSelectedValue();
      if ( sel == ".." )
	updir();
      else
	setup(currentDir + sel);
    }
    if ( e.getSource() == objList ) {
      selectedObject = (String)objList.getSelectedValue();
      if ( e.getClickCount() >= 2 ) {
	done = true;
	dispose();
      } else
	setInfo();
    }
	
  }

  public void mouseEntered(MouseEvent e) {}
  public void mouseExited(MouseEvent e) {}
  public void mousePressed(MouseEvent e) {}
  public void mouseReleased(MouseEvent e) {}

}
