
package JRunMC;

import javax.swing.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

public class ObjectFrame extends JFrame
                         implements ActionListener, MouseListener {

  SetupTheEG owner;
  String object = "";
  String name = "";
  String pclass = "";
  String description = "";
  String info = "";
  JList parameterList;
  JList switchList;
  JList referenceList;
  JList commandList;
  PopCardPanel interfaces = new PopCardPanel();
  JMenuItem open = new JMenuItem("Open");
  JMenuItem close = new JMenuItem("Close");
  JMenuItem classDescription = new JMenuItem("Class Description");

  public ObjectFrame(SetupTheEG frame, String obj) {
    name = obj.substring(obj.lastIndexOf("/") + 1);
    setTitle(name);
    object = obj;
    owner = frame;
    setup();
    parameterList.addMouseListener(this);
    addCard("Parameters:", parameterList);
    addCard("Switches:", switchList);
    addCard("References:", referenceList);
    addCard("Commands:", commandList);
    JPanel north = new JPanel(new GridLayout(2, 1, 2, 2));
    north.setBorder(BorderFactory.createEmptyBorder(2,2,2,2));
    north.add(new JLabel("Object '" + name + "' of class " + pclass,
			 JLabel.CENTER));
    north.setSize(400, 100);
    if ( interfaces.getItemCount() > 0 ) {
      getContentPane().setLayout(new BorderLayout());
      getContentPane().add(interfaces, BorderLayout.CENTER);
      north.add(new JLabel("with the following interfaces:",
			   JLabel.CENTER));
      getContentPane().add(north, BorderLayout.NORTH);
      setSize(400,300);
    } else {
      north.add(new JLabel("has no interfaces.",
			   JLabel.CENTER));
      getContentPane().add(north);
      setSize(400,80);
    }

    JMenuBar menubar = new JMenuBar();
    JMenu file = new JMenu("File");
    open.addActionListener(this);
    file.add(open);
    close.addActionListener(this);
    file.add(close);
    menubar.add(file);
    JMenu show = new JMenu("Show");
    classDescription.addActionListener(this);
    show.add(classDescription);
    menubar.add(show);
    setJMenuBar(menubar);

    setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
    setVisible(true);
    
  }

  public String fullName() {
    return object;
  }

  private void addCard(String name, JList list) {
    if ( list == null ) return;
    interfaces.addCard(name, new JScrollPane(list));
  }

  private void setup() {
    LinkedList ret = owner.exec("fulldescribe " + object);
    if ( ret.size() > 0 ) ret.remove(0);
    if ( ret.size() > 0 ) pclass = (String)ret.remove(0);
    String line = "";
    while ( ret.size() > 0 &&
	    !(line = (String)ret.remove(0)).equals("Interfaces:") )
      description += line;
    Vector parameters = new Vector();
    Vector switches = new Vector();
    Vector references = new Vector();
    Vector commands = new Vector();
    while ( ret.size() > 0 ) {
      line = (String)ret.remove(0);
      if ( line.length() == 0 ) continue;
      switch ( line.charAt(0) ) {
      case 'P':
	parameters.add(line.substring(3));
	break;
      case 'S':
	switches.add(line.substring(3));
	break;
      case 'C':
	commands.add(line.substring(3));
	break;
      case 'R':
	references.add(line.substring(line.indexOf('>') + 2));
	break;
      case 'V':
	if ( line.charAt(1) == '<' )
	  references.add(line.substring(line.indexOf('>') + 2));
	else
	  parameters.add(line.substring(3));
      }
    }
    if ( parameters.size() > 0 ) parameterList = new JList(parameters);
    if ( switches.size() > 0 ) switchList = new JList(switches);
    if ( references.size() > 0 ) referenceList = new JList(references);
    if ( commands.size() > 0 ) commandList = new JList(commands);
  }

  public void dispose() {
    super.dispose();
    owner.remove(object);
  }

  private void showParameter() {
    String par = (String)parameterList.getSelectedValue();
    if ( par == null ) return;
    LinkedList ret = owner.exec("fulldescribe " + object + ":" + par);
    if ( ret.size() == 0 ) return;
    par = (String)ret.getFirst();
    boolean integer = false;
    if ( par.charAt(1) == 'i' ) integer = true;
    else if ( par.charAt(1) == 's' ) return;
    new Parameter(owner, this, ret, integer);
  }

  public void actionPerformed(ActionEvent e) {
    if ( e.getSource() == open ) 
      owner.openObject(object.substring(0, object.lastIndexOf('/') + 1));
    else if ( e.getSource() == close )
      dispose();
    else if ( e.getSource() == classDescription )
      JOptionPane.showMessageDialog(this, description, pclass,
				    JOptionPane.PLAIN_MESSAGE);
  }

  public void mouseClicked(MouseEvent e) {
    if ( e.getSource() == parameterList && e.getClickCount() >= 2 ) {
      showParameter();
    }
  }

  public void mouseEntered(MouseEvent e) {}
  public void mouseExited(MouseEvent e) {}
  public void mousePressed(MouseEvent e) {}
  public void mouseReleased(MouseEvent e) {}

}

