C
C
      SUBROUTINE DIFKIN(XMP1,XMP2,TT,PMOM1,PMOM2,IREJ)
C**********************************************************************
C
C     calculation of diffractive kinematics
C
C     input:    XMP1         mass of outgoing particle system 1 (GeV)
C               XMP2         mass of outgoing particle system 2 (GeV)
C               TT           momentum transfer    (GeV**2, negative)
C
C     output:   PMOM1(5)     four momentum of outgoing system 1
C               PMOM2(5)     four momentum of outgoing system 2
C               IREJ         0    kinematics consistent
C                            1    kinematics inconsistent
C
C**********************************************************************
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      SAVE
C
      DIMENSION PMOM1(5),PMOM2(5)
      PARAMETER (NMAXD=100)
      COMMON /DEBUG/ IDEB(NMAXD),KSPOM,KHPOM,KSREG,KHDIR,KACCEP,
     &  KSTRG,KHTRG,KSLOO,KHLOO,KSDPO,KHDPO,KEVENT,KSOFT,KHARD
C  CM system of diffraction
      COMMON /DIFCMS/ ECMD,PCMD,PMASSD(2),PVIRTD(2),GAMBED(4),
     &                SIDD,CODD,SIFD,COFD,PDCMS(4,2),NPOSD(2)
C
      PARAMETER(EPS  = 1.D-10,
     &          IONE = 1,
     &          PI   = 3.14159265359D0,
     &          PI2  = 6.28318530718D0,
     &          DEPS = 0.001,
     &          ZERO = 0.D0)
C
C  debug output
      IF(IDEB(49).GT.10) THEN
        WRITE(6,'(1X,A,/5X,5E12.4)')
     &    'DIFKIN:DEBUG:ECMD,PCMD,MINI-1,MINI-2,TT:',
     &    ECMD,PCMD,XMP1,XMP2,TT
      ENDIF
C
C  kinematic constraints
      IREJ = 1
      IF(ECMD.LE.1.1D0*(XMP1+XMP2)) RETURN
C  new squared cms momentum
      XMP12 = XMP1**2
      XMP22 = XMP2**2
      SS = ECMD**2
      PCM2 = PCMD**2
      PCMP2 = XLAMB(SS,XMP12,XMP22)**2/(4.D0*SS)
C
C  new longitudinal momentum
      E1I = SQRT(PCM2+PMASSD(1)**2)
      E1F = SQRT(PCMP2+XMP12)
      E2F = SQRT(PCMP2+XMP22)
C
      PLONG = (TT+PCM2+PCMP2-(E1I-E1F)**2)/(2.D0*PCMD)
      PTRAN = PCMP2-PLONG**2
      IF(PTRAN.LT.ZERO) THEN
        IF(IDEB(49).GE.1) THEN
        WRITE(6,'(1X,A)') 'DIFKIN:WARNING:INCONSISTENT KINEMATICS:'
        CALL POPREV(-1)
        WRITE(6,'(1X,A)') 'DIFKIN:DEBUG:INITIAL KINEMATICS:'
        WRITE(6,'(1X,A,/5X,6E12.3)') 'XMP1,XMP2,TT,PCMP,PLONG,PTRANS',
     &    XMP1,XMP2,TT,SQRT(PCMP2),PLONG,SIGN(SQRT(ABS(PTRAN)),PTRAN)
        ENDIF
        PLONG = SQRT(PCMP2)
        PTRAN = ZERO
        TT = 2.D0*PCMD*PLONG+(E1I-E1F)**2-PCM2-PCMP2
        IF(IDEB(49).GE.1) THEN
        WRITE(6,'(1X,A)') 'DIFKIN:DEBUG:ADJUSTED KINEMATICS:'
        WRITE(6,'(1X,A,/5X,6E12.3)') 'XMP1,XMP2,TT,PCMP,PLONG,PTRANS',
     &    XMP1,XMP2,TT,SQRT(PCMP2),PLONG,SQRT(PTRAN)
        ENDIF
        IF(TT.GT.ZERO) THEN
          print *,' DIFKIN:WARNING:TT > 0 ',TT
          IREJ = 1
          RETURN
        ENDIF
      ELSE
        PTRAN = SQRT(PTRAN)
      ENDIF
      XI = PI2*RNDM(PTRAN)
C
C  outgoing momenta in cms system
      PMOM1(4) = E1F
      PMOM1(1) = PTRAN*COS(XI)
      PMOM1(2) = PTRAN*SIN(XI)
      PMOM1(3) = PLONG
      PMOM1(5) = XMP1
C
      PMOM2(4) = E2F
      PMOM2(1) = -PMOM1(1)
      PMOM2(2) = -PMOM1(2)
      PMOM2(3) = -PLONG
      PMOM2(5) = XMP2
      IREJ = 0
C
C  debug output / consistency check
      IF(IDEB(49).GE.0) THEN
C  check kinematics
        XM1 = PMOM1(4)**2-PMOM1(1)**2-PMOM1(2)**2-PMOM1(3)**2
        XM1 = SIGN(SQRT(ABS(XM1)),XM1)
        XM2 = PMOM2(4)**2-PMOM2(1)**2-PMOM2(2)**2-PMOM2(3)**2
        XM2 = SIGN(SQRT(ABS(XM2)),XM2)
        IF((ABS(XM1-XMP1).GT.DEPS).OR.(ABS(XM1-XMP1).GT.DEPS)) THEN
          WRITE(6,'(1X,A,/5X,4E11.4)')
     &      'DIFKIN:WARNING:INCON.MASSES:MINI-1,MOUT-1,MINI-2,MOUT-2',
     &      XMP1,XM1,XMP2,XM2
          CALL POPREV(-1)
        ENDIF
C  output
        IF(IDEB(49).GT.10) THEN
          WRITE(6,'(1X,A,5E11.3,/1X,A,5E11.3)') 
     &      'DIFKIN:DEBUG:P1',PMOM1,'             P2',PMOM2
        ENDIF
      ENDIF
      END
