C
C
      SUBROUTINE MASSAD(IFLMO,IFL1,IFL2,PMASS,XMCON,XMOUT,IDPDG,IDBAM)
C***********************************************************************
C
C    fine correction of low mass strings to mass of corresponding
C    resonance or two particle threshold
C
C    input:     IFLMO         PDG ID of mother particle
C               IFL1,2        requested parton flavours
C                             (not used at the moment)
C               PMASS         reference mass (mass of mother particle)
C               XMCON         conjecture of mass
C
C    output:    XMOUT         output mass (adjusted input mass)
C                             moved ot nearest mass possible
C               IDPDG         PDG resonance ID
C               IDBAM         BAMJET resonance ID
C
C**********************************************************************
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      SAVE
C
      PARAMETER (ZERO   =  0.D0,
     &           ONE    =  1.D0,
     &           TWO    =  2.D0,
     &           OHALF  =  0.5D0,
     &           DEPS   =  1.D-8)
      PARAMETER (NMAXD=100)
      COMMON /DEBUG/ IDEB(NMAXD),KSPOM,KHPOM,KSREG,KHDIR,KACCEP,
     &  KSTRG,KHTRG,KSLOO,KHLOO,KSDPO,KHDPO,KEVENT,KSOFT,KHARD
C
      CHARACTER*8 MDLNA
      COMMON /MODELS/ MDLNA(50),ISWMDL(50),PARMDL(200),IPAMDL(200)
C  particle properties
      CHARACTER*8 ANAME
      COMMON/POPART/ANAME(187),AM(187),GA(187),TAU(187),ICH(187),
     &IBAR(187),K1(187),K2(187)
C  decay channels
      COMMON/DECAYC/ ZKNAME(550),NZK(550,3),WT(550)
C
      DIMENSION XWG(10),RMA(10),RGA(10),RWG(10),IRPDG(10),IRBAM(10)
C
      XMINP = XMCON
      IDPDG = 0
      IDBAM = 0
      XMOUT = XMINP
C  resonance treatment activated?
      IF(ISWMDL(23).EQ.0) RETURN
C
      CALL DIFRES(IFLMO,IFL1,IFL2,IRPDG,IRBAM,RMA,RGA,RWG,LISTL)
      IF(LISTL.LT.1) THEN
        IF(IDEB(7).GE.2) WRITE(6,'(1X,A,3I7)')
     &    'MASSAD:WARNING:NO RESONANCES (IFMO,IF1,IF2)',
     &    IFLMO,IFL1,IFL2
        GOTO 50
      ENDIF
C  mass small?
      PMASSL = (PMASS+0.15D0)**2
      XMINP2 = XMINP**2
C  determine resonance probability
      DM2 = 1.1D0
      RPROB = (PMASSL+DM2)*(XMINP2-PMASSL)/(DM2*XMINP2)
      IF(RPROB.LT.RNDM(PMASSL)) THEN
C  sample new resonance
        XWGSUM = ZERO
        DO 100 I=1,LISTL
          XWG(I) = RWG(I)/RMA(I)**2
          XWGSUM = XWGSUM+XWG(I)
 100    CONTINUE
C
        ITER = 0
 150    CONTINUE
        ITER = ITER+1
        IF(ITER.GE.5) THEN
          IDBAM = 0
          IDPDG = 0
          XMOUT = XMINP
          GOTO 50
        ENDIF
C
        I = 0
        XI = XWGSUM*RNDM(XMOUT)
 200    CONTINUE
          I = I+1
          XWGSUM = XWGSUM-XWG(I)
        IF((XI.LT.XWGSUM).AND.(I.LT.LISTL)) GOTO 200
        IDPDG = IRPDG(I)
        IDBAM = IRBAM(I)
        GARES = RGA(I)
        XMRES = RMA(I)
        XMRES2 = XMRES**2
C  sample new mass (relativistic Breit-Wigner shape)
        ALO = ATAN((PMASSL+DM2-XMRES2)/(XMRES*GARES))
        AHI = ATAN((5.D0-XMRES2)/(XMRES*GARES))
        XI = (AHI-ALO)*RNDM(XMRES)+ALO
        XMOUT = XMRES*GARES*TAN(XI)+XMRES2
        XMOUT = SQRT(XMOUT)
C  check mass for decay
        AMDCY = TWO*XMRES
        DO 250 IK=K1(IDBAM),K2(IDBAM)
          AMSUM = ZERO
          DO 275 I=1,3
            IF(NZK(IK,I).NE.0) AMSUM = AMSUM+AM(NZK(IK,I))
 275      CONTINUE
          AMDCY = MIN(AMDCY,AMSUM)
 250    CONTINUE
        IF(AMDCY.GE.XMOUT) GOTO 150
C  debug output
        IF(IDEB(7).GE.10)
     &    WRITE(6,'(1X,A,/1X,3I6,2E10.3,2I7,2E10.3)')
     &    'MASSAD:DEBUG:IFMO,IF1,IF2,XMCON,XMOUT,IDPDG,IDBAM,RMA,RGA',
     &    IFLMO,IFL1,IFL2,XMCON,XMOUT,IDPDG,IDBAM,RMA(I),RGA(I)
        RETURN
      ENDIF
C
 50   CONTINUE
C  debug output
      IF(IDEB(7).GE.15)
     &  WRITE(6,'(1X,A,/1X,3I6,2E10.3)')
     &    'MASSAD:DEBUG:CHAIN SAMPLED: IFMO,IF1,IF2,XMCON,XMOUT',
     &    IFLMO,IFL1,IFL2,XMCON,XMOUT
      END
