C
C
      SUBROUTINE SDECAY(NPOS,ISP,ILEV)
C**********************************************************************
C
C     decay of single resonance of COMMON HEPEVS:
C       decay in helicity frame according to polarization, isotropic 
C       decay and decay with limited transverse phase space possible
C
C     ATTENTION:
C     reference to particle number of BAMJET has to existc
C
C     input:   NPOS    position in /HEPEVS/
C              ISP     0  decay according to phase space
C                      1  decay according to transversal polarization
C                      2  decay according to longitudinal polarization
C                      3  decay with limited phase space
C              ILEV    decay mode to use
C                      1 strong + weak decays
C                      2 only strong decays
C                      3 strong decays, weak decays for charmed part.
C                        and tau leptons
C                      negative: remove mother resonance after decay
C
C     output:  COMMON HEPEVS/E final particles according to decay mode
C
C**********************************************************************
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      SAVE
C
      PARAMETER(NTAB = 20,
     &          EPS  = 1.D-15,
     &          ONE  = 1.D0,
     &          OHALF= 0.5D0,
     &          PI   = 3.14159265359D0,
     &          PI2  = 6.28318530718D0,
     &          DEPS = 1.D-10,
     &          ZERO = 0.D0)
C
      PARAMETER (NMAXD=100)
      COMMON /DEBUG/ IDEB(NMAXD),KSPOM,KHPOM,KSREG,KHDIR,KACCEP,
     &  KSTRG,KHTRG,KSLOO,KHLOO,KSDPO,KHDPO,KEVENT,KSOFT,KHARD
C
      CHARACTER*8 MDLNA
      COMMON /MODELS/ MDLNA(50),ISWMDL(50),PARMDL(200),IPAMDL(200)
C
      PARAMETER (NMXHEP=2000)
      COMMON/HEPEVS/NEVHEP,NHEP,ISTHEP(NMXHEP),IDHEP(NMXHEP),
     &JMOHEP(2,NMXHEP),JDAHEP(2,NMXHEP),PHEP(5,NMXHEP),VHEP(4,NMXHEP)
      COMMON/HEPEVE/IMPART(NMXHEP),IPHIST(2,NMXHEP),ICOLOR(2,NMXHEP)
C  particle properties
      CHARACTER*8 ANAME
      COMMON/POPART/ANAME(187),AM(187),GA(187),TAU(187),ICH(187),
     &IBAR(187),K1(187),K2(187)
C  decay channels
      COMMON/DECAYC/ ZKNAME(550),NZK(550,3),WT(550)
C  final particles
      COMMON /DECPHS/ ECM(3),PCM(3),COD(3),COF(3),SIF(3)
C
      DIMENSION WGHD(20),KCH(20),ID(3),IFL(3)
C
      IDBAM = IMPART(NPOS)
      IMODE = ABS(ILEV)
      IF(IDEB(36).GE.15) WRITE(6,'(1X,A,3I5)') 
     &  'SDECAY:DEBUG:NPOS,ISP,ILEV',NPOS,ISP,ILEV
C  particle stable?
      IF(IMODE.EQ.1) THEN
        IF((IDBAM.EQ.135).OR. (IDBAM.EQ.136)) GOTO 400
        IF((IDBAM.GE.  1).AND.(IDBAM.LE.  7)) GOTO 400
      ELSE IF(IMODE.EQ.2) THEN
        IF((IDBAM.GE.  1).AND.(IDBAM.LE. 25)) GOTO 400
        IF((IDBAM.GE. 97).AND.(IDBAM.LE.103)) GOTO 400
        IF((IDBAM.GE.115).AND.(IDBAM.LE.122)) GOTO 400
        IF((IDBAM.GE.131).AND.(IDBAM.LE.136)) GOTO 400
        IF(IDBAM.EQ.109)                      GOTO 400
        IF((IDBAM.GE.137).AND.(IDBAM.LE.160)) GOTO 400
      ELSE IF(IMODE.EQ.3) THEN
        IF((IDBAM.GE.  1).AND.(IDBAM.LE. 23)) GOTO 400
        IF((IDBAM.GE. 97).AND.(IDBAM.LE.103)) GOTO 400
        IF((IDBAM.EQ.109).AND.(IDBAM.EQ.115)) GOTO 400
        IF((IDBAM.GE.133).AND.(IDBAM.LE.136)) GOTO 400
      ELSE
        WRITE(6,'(/1X,A,I5)') 'SDECAY:ERROR:INVALID MODE',IMODE
        CALL POABRT
      ENDIF
C  decay products
      K = 0
      WGSUM = ZERO
      AMIST = PHEP(5,NPOS)
      DO 100 I=K1(IDBAM),K2(IDBAM)
        AMSUM = ZERO
        DO 200 L=1,3
          ID(L) = NZK(I,L)
          IF(ID(L).NE.0) THEN
            AMSUM = AMSUM+AM(ID(L))
          ENDIF
 200    CONTINUE
        IF(AMSUM.LT.AMIST) THEN
          K = K+1
          WGHD(K) = WT(I)
          KCH(K) = I
        ENDIF
 100  CONTINUE
      IF(K.EQ.0)THEN
        WRITE(6,'(/1X,A,I6,3E12.4)')
     &    'SDECAY:WARNING:TOO SMALL PARTICLE MASS (NPOS,MA,DCYM)',
     &    NPOS,AMIST,AMSUM
        CALL POPREV(0)
        RETURN
      ENDIF
C  sample new decay channel
      XI = (RNDM(AMSUM)-EPS)*WGSUM
      K = 0
      WGSUM = ZERO
 500  CONTINUE
        K = K+1
        WGSUM = WGSUM+WGHD(K)
      IF(XI.GT.WGSUM) GOTO 500
      IK = KCH(K)
      ID(1) = NZK(IK,1)
      ID(2) = NZK(IK,2)
      ID(3) = NZK(IK,3)
C  decay possible?
      IF(ID(2).LT.1) RETURN
C  rotation
      PTOT = SQRT(PHEP(1,NPOS)**2+PHEP(2,NPOS)**2+PHEP(3,NPOS)**2)
      CXS = PHEP(1,NPOS)/PTOT
      CYS = PHEP(2,NPOS)/PTOT
      CZS = PHEP(3,NPOS)/PTOT
C  boost
      GBET = PTOT/AMIST
      GAM = PHEP(4,NPOS)/AMIST
C
      IF(ID(3).EQ.0) THEN
C  two particle decay
        CALL SDECY2(AMIST,AM(ID(1)),AM(ID(2)),ISP)
      ELSE
C  three particle decay
        CALL SDECY3(AMIST,AM(ID(1)),AM(ID(2)),AM(ID(3)),ISP)
      ENDIF
C
      IF(ILEV.LT.0) THEN
        IF(NHEP.NE.NPOS) THEN
          WRITE(6,'(/1X,A,2I5)')
     &    'SDECAY:ERROR:CANNOT REMOVE RESONANCE (NPOS,NHEP)',NPOS,NHEP
          CALL POABRT
        ENDIF
        IMO1 = JMOHEP(1,NPOS)
        IMO2 = JMOHEP(2,NPOS)
        NHEP = NHEP-1
      ELSE
        IMO1 = NPOS
        IMO2 = 0
      ENDIF
C  back transformation and registration
      DO 300 I=1,3
        IF(ID(I).GT.0) THEN
          CALL TRAFO(GAM,GBET,CXS,CYS,CZS,COD(I),COF(I),SIF(I),
     &      PCM(I),ECM(I),PTOT,CX,CY,CZ,EE)
          XX = PTOT*CX
          YY = PTOT*CY
          ZZ = PTOT*CZ
          CALL REGPAR(1,IFL(I),ID(I),IMO1,IMO2,XX,YY,ZZ,EE,0,0,0,0,
     &                IPOS,1)
        ENDIF
 300  CONTINUE
C
 400  CONTINUE
C  debug output
      IF(IDEB(36).GE.20) THEN
        WRITE(6,'(2(/1X,A))') 'SDECAY:DEBUG:COMMON HEPEVS',
     &                        '=========================='
        CALL POPREV(0)
      ENDIF
      END
