
***********************************************************************
      SUBROUTINE FRAGPJ (IBP, IDIFP, ISCN,IERR)
      
      IMPLICIT NONE

************************************************************************
*
*     Purpose:    Take hadronic mass at p vertex and let it decay
*     --------    via JETSET
*
*     Input:      IBP:    Number of beam proton in /CPROD/
*     ------      IDIFP:  Number of diffractive state in /CPROD/
*                 from common block /CGDIF/:
*                 IBEAMP
*                 from common block /CPROD/:
*                 NPART, PPCMS8
*                 from common block /CMASS/:
*                 DMP, DMPI0
*
*     Output:     ISCN:   Number of scattered nucleon /CPROD/
*     -------     to common block /CPROD/:
*                 NPART, PPCMS8, ITYPE, MOHEP, IDAHEP, ISTAT
*
*     Called by:  GENEVT
*     ----------
*
*     Author:     Benno List, 17.1.94
*     -------
*
*     Changed by:
*     -----------
*
*     Calling:    From JETSET 7.3:
*     --------    LU1ENT, LUGIVE, LUJOIN, LULIST, LUEXEC, PLU, KLU
*
************************************************************************
#include "cmass.inc"
#include "cgdif.inc"
#include "cprod.inc"
#include "zdludat1.inc"
#include "ntblock.inc"
      
      INTEGER IQ,IQQ,IERR,NTRIAL,I,NERR,N,KS,J,KLU,IDIFP,
     +        ISCN,NPROT,NNEUTR,IER,NPI,NP,NM,N0,IBP
      REAL*8 PQQ (5), PQ (5), PPOM (5), R, R1 
      REAL A, PLU

C_la
      integer la_ig
      real*8 la_pg(5),la_prem(5)
      integer la_iarr132(3) /1, 3, 2/
C_la     

      CHARACTER*100 C100
      INTEGER IARR12 (2) /1, 2/


      DATA NERR /0/

      EXTERNAL LUDATA
      IERR=0

    
        nprot=0
        nneutr=0

ccc      PRINT *, '--- FRAGPJ ---'
ccc      PRINT *, 'IBP   = ', IBP
ccc      PRINT *, 'IDIFP = ', IDIFP
ccc      PRINT *, 'NPART = ', NPART

      IF (PPCMS8 (5, IDIFP) .LT. DMP+DMPI0) THEN
        PRINT *, '### FRAGPJ error: not enough energy! '
ccc        PRINT *, 'IBP:   ', IBP
ccc        PRINT *, 'IDIFP: ', IDIFP
ccc        PRINT *, 'PPCMS8 (5, IDIFP): ', PPCMS8 (5, IDIFP)
ccc        PRINT *, 'DMP + DMPI0:       ', DMP+DMPI0
        PRINT*, ' FRAGPJ: Not enough energy!'
          IERR=1

      RETURN
      END IF
*
* Split proton into q - qq system: choose q and qq codes
* The pomeron couples with the same strength to u and d quarks.
* ud_1 and uu_1 are spin triplets, ud_0 is a spin singulett.
* Therefore q-qq is 1/7 u-ud_0, 3/7 u-ud_1, and 3/7 d-uu_1.
*
   
      CALL myRANLUX(A,1)
      r1=dble(a)
      R = 7.0*r1
      IF (R .LE. 1.0) THEN

        IQ  = SIGN (2, IBEAMP)       !u
        IQQ = SIGN (2101, IBEAMP)    !ud spin 0
      ELSE IF (R .LE. 4.0) THEN
        IQ  = SIGN (2, IBEAMP)       !u
        IQQ = SIGN (2103, IBEAMP)    !ud spin 1
      ELSE
        IQ  = SIGN (1, IBEAMP)       !d
        IQQ = SIGN (2203, IBEAMP)    !uu spin 1
      END IF

C_la  add gluon
        la_ig = SIGN (21, IBEAMP)
C_la

C     PRINT *, 'IQ:  ', IQ,  ', mass: ', ULMASS (IQ)
C     PRINT *, 'IQQ: ', IQQ, ', mass: ', ULMASS (IQQ)

* Calculate the 4-vector of the proton remnant and of the struck quark
* To avoid numerical problems, calculate pomeron momentum from
* proton side
      DO 10, I = 1, 4
        PPOM (I) = PPCMS8 (I, IDIFP) - PPCMS8 (I, IBP)
 10   CONTINUE
      PPOM (5) = -SQRT(PPOM(1)**2+PPOM (2)**2+PPOM(3)**2-PPOM(4)**2)
C calculate 4-momentum of quark and diquark

C_la      CALL SPLITP (PPCMS8 (1, IBP), PPOM, PQ, PQQ, IER)
          CALL SPLITP (PPCMS8 (1, IBP), PPOM, la_pg, la_prem, IER)
          do i=1,5
           pq(i) = 1./3. * la_prem(i)
           pqq(i) = 2./3. * la_prem(i)
          enddo
C_la 
      NTRIAL = 0
 12   CONTINUE
        NTRIAL = NTRIAL+1


        CALL LU1ENT (1, IQQ, SNGL (PQQ (4)), 0.0, 0.0)
        DO 15, I = 1, 5
          WRITE (C100, 999) 1, I, SNGL (PQQ (I))
          CALL LUGIVE (C100)
  15    CONTINUE
    
        CALL LU1ENT (2, IQ, SNGL (PQ (4)), 0.0, 0.0)
        DO 16, I = 1, 5
          WRITE (C100, 999) 2, I, SNGL (PQ (I))
          CALL LUGIVE (C100)
  16    CONTINUE

C_la
        CALL LU1ENT (3, la_ig, SNGL (la_pg (4)), 0.0, 0.0)
        DO 17, I = 1, 5
           WRITE (C100, 999) 3, I, SNGL (la_pg (I))
           CALL LUGIVE (C100)
  17    CONTINUE

C_la

 999    FORMAT ('P(',I4,',',I1,')=',F9.4)

* Define string between qq and q


            CALL LUJOIN (2, IARR12)
            CALL LUJOIN (3, la_IARR132)
   

C       CALL LULIST (1)

* Perform fragmentation with JETSET
        CALL LUEXEC

* Check for possible Error
      
        IF (MSTU (24) .NE. 0) THEN
          PRINT*, '  FRAGPJ: JETSET error'
          NERR = NERR + 1
          IF (NERR .LT. 100) THEN
            PRINT *, ' *** FRAGPJ: JETSET error', MSTU (24)
          END IF
        ELSE IF (MSTU (28) .NE. 0) THEN
          PRINT*, ' FRAGPJ: JETSET warning'
          NERR = NERR + 1
          IF (NERR .LT. 100) THEN
            PRINT *, ' *** FRAGPJ: JETSET warning', MSTU (28)
          END IF
        END IF

      IF (NTRIAL .LE. 10 .AND.
     +    (MSTU (24) .NE. 0 .OR. MSTU (28) .NE. 0)) GO TO 12

      IF (MSTU (24) .NE. 0) THEN
        PRINT *, ' *** FRAGPJ: Cannot prevent JETSET error'
        PRINT *, '     Mass: ', PPCMS8 (5, IDIFP)

      ELSE IF (MSTU (28) .NE. 0) THEN
        PRINT *, ' *** FRAGPJ: Cannot prevent JETSET warning'
        PRINT *, '     Mass: ', PPCMS8 (5, IDIFP)

      END IF

C     CALL LULIST (1)

*
* Update /CPROD/
*

      N = KLU (0, 1)
      IF (NPART + N .GT. NPARTMAX) THEN
        PRINT *, '### FRAGPJ error:too many particles for PPART!'

        PRINT *, 'Event record will not conserve 4-momentum!'
        PRINT *, 'NPART =    ', NPART
        PRINT *, 'N =        ', N
        PRINT *, 'NPARTMAX = ', NPARTMAX
        N = NPARTMAX - NPART - 3
      END IF

      ISTAT (IDIFP) = 2
      IDAHEP (1, IDIFP) = NPART + 1
      IDAHEP (2, IDIFP) = NPART + 2

      ISCN = 0
      DO 30, I = 1, N
        ITYPE (NPART + I) = KLU (I, 2) 
        KS = KLU (I, 1)
        IF (KS .EQ. 0) THEN
*         null entry
          ISTAT (NPART + I) = -1
        ELSE IF (KS .GE. 1 .AND. KS .LE. 5) THEN
*         undecayed particle or unfragmented jet
          ISTAT (NPART + I) = 1
        ELSE IF (KS .GE. 11 .AND. KS .LE. 15) THEN
*         decayed particle or fragmented jet
          ISTAT (NPART + I) = 2
        ELSE 
*         documentation line
          ISTAT (NPART + I) = 3
        END IF
        PPCMS8 (5, NPART+I) = DBLE (PLU (I, 5))
        PPCMS8 (4, NPART+I) = PPCMS8 (5, NPART+I)**2
        DO 20, J = 1, 3
         PPCMS8 (J, NPART+I) = DBLE (PLU (I, J))
         PPCMS8 (4,NPART+I)=PPCMS8(4,NPART+I)+PPCMS8(J,NPART+I)**2
 20     CONTINUE
        PPCMS8 (4, NPART+I) = SQRT (PPCMS8 (4, NPART+I))
        IF ((SIGN (ITYPE (NPART+I), IBEAMP) .EQ. 2212 .OR.      !proton
     +       SIGN (ITYPE (NPART+I), IBEAMP) .EQ. 2112 .OR.      !neutron
     +       SIGN (ITYPE (NPART+I), IBEAMP) .EQ. 3122 .OR.      !lambda0
     +       SIGN (ITYPE (NPART+I), IBEAMP) .EQ. 3222 .OR.      !sigma+
     +       SIGN (ITYPE (NPART+I), IBEAMP) .EQ. 3212 .OR.      !sigma0
     +       SIGN (ITYPE (NPART+I), IBEAMP) .EQ. 3112 .OR.      !sigma-
     +       SIGN (ITYPE (NPART+I), IBEAMP) .EQ. 3322 .OR.      !Xi0
     +       SIGN (ITYPE (NPART+I), IBEAMP) .EQ. 3312 .OR.      !Xi-
     +       SIGN (ITYPE (NPART+I), IBEAMP) .EQ. 3334)          !omega-
     +      .AND. ISTAT (NPART + I) .EQ. 1) THEN
          IF (ISCN .EQ. 0) THEN
            ISCN = NPART + I
          ELSE IF (PPCMS8 (3, NPART+I) .GT. PPCMS8 (3, ISCN)) THEN
            ISCN = NPART + I
          END IF
        END IF


        if(SIGN (ITYPE (NPART+I), IBEAMP) .EQ. 2212) then
        nprot=nprot+1
        endif
        if(SIGN (ITYPE (NPART+I),IBEAMP).EQ. 2112) then 
        nneutr=nneutr+1
        endif

        hnprot = nprot
        hnneutr = nneutr



c       CALL PR5S('PLU ',PLU(I,1),PLU(I,2),PLU(I,3),PLU(I,4),PLU(I,5))
c       CALL PR5V8 ('PPCMS8', PPCMS8 (1, NPART+I))
        IF (KLU (I, 4) .EQ. 0) THEN
          IDAHEP (1, NPART + I) = 0
          IDAHEP (2, NPART + I) = 0
        ELSE
          IDAHEP (1, NPART + I) = NPART + KLU (I, 4)
          IDAHEP (2, NPART + I) = NPART + KLU (I, 5)
        END IF
        MOHEP (2, NPART + I) = 0
        IF (KLU (I, 3) .EQ. 0) THEN
          MOHEP (1, NPART + I) = IDIFP
        ELSE
          MOHEP (1, NPART + I) = NPART + KLU (I, 3)
          IF (ITYPE(NPART+I).GE.91.AND.ITYPE(NPART+I).LE.93) THEN
* for fragmentation string look for second mother
            DO 26, J = KLU (I, 3)+1, I-1
              IF (KLU (J, 4) .EQ. I) MOHEP (2, NPART+I) = NPART+J
 26         CONTINUE
          END IF
        END IF

       if((ITYPE (NPART+I)) .EQ. 211) then
        np=np+1
        endif

       if((ITYPE (NPART+I)) .EQ. -211) then
        nm=nm+1
        endif

        if((ITYPE (NPART+I)) .EQ. 111) then
        n0=n0+1
        endif
        npi = np + nm + n0

        hnpi = npi
        hnp = np
        hnm = nm
        hn0 = n0
 30   CONTINUE

      IF (ISCN .EQ. 0) THEN
        PRINT *, 'FRAGPJ WARNING: No scattered nucleon found!'
        PRINT *, '     I     ITYPE    ISTAT'
        DO 40, I = 1, N
ccc          PRINT *, NPART+I, ITYPE(NPART+I), ISTAT (NPART + I)
 40     CONTINUE
      END IF

      NPART = NPART + N

* Print control output:

ccc      PRINT *, 'FRAGPJ: Result'
ccc      PRINT *, 'ISCN: ', ISCN
c     CALL PR5V8 ('p_pdif', PPCMS8 (1, IDIFP))
c     CALL PR5S  ('JETSET',PLU(0,1),PLU(0,2),PLU(0,3),PLU(0,4),PLU(0,5))
      
      RETURN
      END


