// * Copyright (C) 2002-2004 Sergei Chekanov (ANL) chekanov@mail.desy.de 
// * This file is part of the RunMC project
// * 
// * Permission is hereby granted, free of charge, to any person obtaining a copy
// * of this software and associated documentation files (the "Software"), to
// * deal in the Software without restriction, including without limitation the
// * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// * sell copies of the Software, and to permit persons to whom the Software is
// * furnished to do so, subject to the following conditions:
// *
// * The above copyright notice and this permission notice shall be included in
// * all copies or substantial portions of the Software.
// * 
// * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// * SERGEI  CHEKANOV BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
// * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// * SOFTWARE.
// * 
// * Except as contained in this notice, the name of Sergei Chekanov shall
// * not be used in advertising or otherwise to promote the sale, use or other
// * dealings in this Software without prior written authorization from
// * Sergei Chekanov.
// * 
// get all calculated value into the histogram
using namespace std;
#include<TROOT.h>
#include<iostream>
#include<heplist.h>

// CLHEP
#include "CLHEP/config/CLHEP.h"
#include "CLHEP/Vector/LorentzVector.h"
#include "CLHEP/Vector/Boost.h"
#include "CLHEP/Vector/Rotation.h"


extern heplist hep;      // get particles after the cuts


Int_t  afill( char * type, char * nnn, Double_t* getval, Long_t nc1=-1, Long_t nc2=-1);


// ----------------------- Fill stable hadrons 
Int_t  afill( char * type, char * nnn, double* getval, long nc1, long nc2) {

// Input:
// type="particle" - get varibales for separate particles or events
// type="jet"      - get informations on jets only 
// nnn              - name of histogram
// nc1,nc2  - No of current particle 1 and particle 2 (for 2-particle density)
//            if not filled - assumes -1
// 
// Output:
// getval - value to fill histogram

   *getval = -99999;

// fill first varibles charcterising events ------------------------ 
   if (nc1==-1 && nc2==-1) {
     if (strcmp(nnn,"N(tot)") == 0)  {*getval=hep.PP.size();}
// PTtot
      else if (strcmp(nnn,"PTtot") == 0)  {*getval=hep.PTtot;}
// PTtot
      else if (strcmp(nnn,"PZtot") == 0)  {*getval=hep.PZtot;}
//Etot
      else if (strcmp(nnn,"Etot") == 0)  {*getval=hep.Etot;}
      else {
// do not do anything
       return 1; };
   }; // end filling event varibales 


   
// fill first varibles charcterising single particles ------------------------
     if (nc1 != -1 && nc2==-1 && strcmp(type,"particle") == 0) {
// for all particles
     if      (strcmp(nnn,"Px") == 0)  { *getval=hep.PP[nc1].p.px();}
     else if (strcmp(nnn,"Py") == 0)  { *getval=hep.PP[nc1].p.py();}
     else if (strcmp(nnn,"Pz") == 0)  { *getval=hep.PP[nc1].p.pz();}
     else if (strcmp(nnn,"E")  == 0)  { *getval=hep.PP[nc1].p.e();}
     else if (strcmp(nnn,"Perp2")  == 0) { *getval=hep.PP[nc1].p.perp2(); }
     else if (strcmp(nnn,"Perp")  == 0)  { *getval=hep.PP[nc1].p.perp(); }
     else if (strcmp(nnn,"Phi")  == 0)   { *getval=hep.PP[nc1].p.phi(); }
     else if (strcmp(nnn,"Theta")  == 0) { *getval=hep.PP[nc1].p.theta(); }
     else if (strcmp(nnn,"Eta")  == 0)   { *getval=hep.PP[nc1].p.pseudoRapidity(); }
     else if (strcmp(nnn,"Rapidity")  == 0)  {
               if (hep.PP[nc1].p.e()-hep.PP[nc1].p.pz()>0) {
                   *getval=hep.PP[nc1].p.rapidity(); }  }
       else {
       return 1; };
   };


// fill first varibles charcterising 2-particle density (invarinat mass etc) ---------------
   if (nc1 != -1 && nc2 !=-1 && strcmp(type,"particle") == 0) {
// for all particles
   if      (strcmp(nnn,"M12") == 0 )     {
   bool isSL1=hep.PP[nc1].p.isTimelike();
   bool isSL2=hep.PP[nc2].p.isTimelike();
   if (isSL1) { if (isSL2) {
   *getval=hep.PP[nc1].p.invariantMass(hep.PP[nc2].p); };  }; }
//   else if (strcmp(nnn,"M12**2") == 0)  { *getval=hep.PP[nc1].p.invariantMass2(hep.PP[nc2].p);}
// do not do anything
   else {
       return 1;
   };
   }; // end 2-particle densities  



// -------------------- this should be filled by the user ----------------------------------
// fill jet varibales (1-particle densities) 
    if (nc1 == -1 && nc2 ==-1 && strcmp(type,"jet") == 0) {
    if (strcmp(nnn,"Put_here_some_title1") == 0)  {
    } else {
       return 1; 
   }; }; // end 2-particle densities


// fill jet varibales (1-particle densities)
   if (nc1  != -1 && nc2 ==-1 && strcmp(type,"jet") == 0) {
    if (strcmp(nnn,"Put_here_some_title2") == 0)  {
    } else {
       return 1;
   }; }; // end 2-particle densities

// fill jet varibales (1-particle densities)
   if (nc1  != -1 && nc2  !=-1 && strcmp(type,"jet") == 0) {
    if (strcmp(nnn,"Put_here_some_title3") == 0)  {
    } else {
       return 1;
   }; }; // end 2-particle densities


      return 0;
}
