// * Copyright (C) 2002-2004 Sergei Chekanov (ANL) chekanov@mail.desy.de 
// * This file is part of the RunMC project
// * 
// * Permission is hereby granted, free of charge, to any person obtaining a copy
// * of this software and associated documentation files (the "Software"), to
// * deal in the Software without restriction, including without limitation the
// * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
// * sell copies of the Software, and to permit persons to whom the Software is
// * furnished to do so, subject to the following conditions:
// *
// * The above copyright notice and this permission notice shall be included in
// * all copies or substantial portions of the Software.
// * 
// * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
// * SERGEI  CHEKANOV BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
// * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// * SOFTWARE.
// * 
// * Except as contained in this notice, the name of Sergei Chekanov shall
// * not be used in advertising or otherwise to promote the sale, use or other
// * dealings in this Software without prior written authorization from
// * Sergei Chekanov.
// * 
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string.h>
#include <time.h>
using namespace std;

//******************************************************************************

int decom_inverse ( char *file_in_name, char *file_out_name, char comment[] )

//******************************************************************************
//
//  Purpose:
//
//    handles a single file.
//
//  Discussion:
//
//    The routine reads a line, and if it does not begin with 
//    'comment' it copies
//    it to the output file.
//
//  Modified:
//
//    08 December 2003
//
//  Author:
//
//    John Burkardt
//    S. Chekanov (modified) 
//
//  Parameters:
//
//    Input, char FILE_IN_NAME[], the name of the input file.
//    Input, char FILE_OUT_NAME[], the name of the output file.
//    Input, char comment - any character. If line begins with this
//    character -  this line will be removed;   
//  
{
  int comment_count;
  ifstream file_in;
  ofstream file_out;
  char line[255];
//
// Open the input file.
//
  file_in.open ( file_in_name );

  if ( !file_in ) 
  {
    cout << "\n";
    cout << "HANDLE - Fatal error!\n";
    cout << "  Cannot open \"" << file_in_name << "\"\n";
    return 1;
  }
//
// Open the output file.
//
  file_out.open ( file_out_name );

  if ( !file_out ) 
  {
    cout << "\n";
    cout << "HANDLE - Fatal error!\n";
    cout << "  Cannot open \"" << file_out_name << "\"\n";
    return 1;
  }

  comment_count = 0;

  while ( 1 )
  {
    file_in.getline ( line, sizeof ( line ) );

    if ( file_in.eof() )
    {
      break;
    }
    if ( *line != *comment )
    {
      comment_count = comment_count + 1;
    }
    else
    {
     file_out << line << "\n";
    }
  }
//
// Close the files.
//
  file_in.close ( );
  file_out.close ( );

//  cout << file_in_name << " contained " << comment_count << " comments.\n";

  return 0;
}
