       Subroutine Boost(bex,bey,bez,p,pnew)
C#*****************************************************************
C#       Subroutine Boost(bex,bey,bez,p,pnew)
C#
C@#  BOOST : Performs Lorentz boosts of given 4-momentum-vector
C#
C#  PURPOSE : Performs Lorentz boosts of given 4-momentum-vector
C#
C# INPUT :
C#   BEX, BEY, BEZ  =
C#        gives the direction and size, beta, of a
C#         Lorentz boost, such that a particle initially at rest
C#       will  have p/E = beta afterwards
C#
C#    P             =  is the vector to be acted upon
C#
C#  OUTPUT :
C#   PNEW           =  is the boosted vector
C#
C# Author: LUROBO in JETSET (modified by N Brook)
C#
C#*****************************************************************
       Implicit Double Precision (D)
       Dimension dp(4), p(4), pnew(4)
       CALL UCOPY(p,pnew,4)
       IF(BEX**2+BEY**2+BEZ**2.GT.1E-20) THEN
C...LORENTZ BOOST (TYPICALLY FROM REST TO MOMENTUM/ENERGY=BETA)
        DBEX=dble(BEX)
        DBEY=dble(BEY)
        DBEZ=dble(BEZ)
        DGA=1D0/SQRT(1D0-DBEX**2-DBEY**2-DBEZ**2)
        DO 130 J=1,4
 130    DP(J)=DBLE(P(J))
        DBEP=DBEX*DP(1)+DBEY*DP(2)+DBEZ*DP(3)
        DGABEP=DGA*(DGA*DBEP/(1D0+DGA)+DP(4))
        PNEW(1)=SNGL(DP(1)+DGABEP*DBEX)
        PNEW(2)=SNGL(DP(2)+DGABEP*DBEY)
        PNEW(3)=SNGL(DP(3)+DGABEP*DBEZ)
        PNEW(4)=SNGL(DGA*(DP(4)+DBEP))
      ENDIF
      RETURN
      END


