      subroutine breit_matrix (q,Llb,Lbl)
* This routine computes the matrix to make the boost from the laboratory into
* the Breit (Llb) and viceversa (Lbl).

      implicit none

* Written by Oscar Gonzalez (Feb. 10th, 2002)

      double precision Llb(4,4),Lbl(4,4)
      double precision q(4)

* The arguments of the routine are (note the double precision!!!)
*
* Input
*          q is the four momentum of the exchanged boson (4 is the 
*            temporal component, that is the energy)
*
* Output
*          Llb is the matrix for the boost from the lab into the Breit
*              To boost a vector vl(4) use 
*                         do i=1,4
*                            vb(i)=0
*                            do j=1,4
*                               vb(i)=vb(i) + Llb(i,j)*vl(j)
*                            enddo
*                         enddo 
*
*         Lbl is the matrix for the boost from the Breit into the lab (the
*             way to boost a vector is the same but changing always l <-> b
*
* NOTE: You can use breit_matrix_e if you prefer to use the fourmomentum
*       of the scattered electron/positron to reconstruct the Breit.
*
      integer i,j
*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
* The expressions used here are the analytical ones.

      Llb(1,1)=sqrt(q(1)**2+q(2)**2) ! qt
      if (Llb(1,1).gt. 0.) then  ! To avoid problems
         Llb(1,2)=sqrt(q(1)**2+q(2)**2+q(3)**2-q(4)**2) ! Q
         Llb(4,1)=q(4)-q(3)

         Llb(1,3)=-Llb(1,1)/Llb(4,1)
         Llb(1,4)=-Llb(1,3)
         Llb(2,1)=q(2)/Llb(1,1)
         Llb(2,2)=-q(1)/Llb(1,1)
         Llb(2,3)=0.0d0
         Llb(2,4)=0.0d0

         Llb(3,1)=-q(1)/Llb(1,2)
         Llb(3,2)=-q(2)/Llb(1,2)
         Llb(3,3)=-q(3)/Llb(1,2)
         Llb(3,4)=q(4)/Llb(1,2)

         Llb(4,2)=Llb(3,2)
         Llb(4,3)=Llb(3,3)-Llb(1,2)/Llb(4,1)
         Llb(4,4)=Llb(3,4)+Llb(1,2)/Llb(4,1)

         Llb(4,1)=Llb(3,1)
         Llb(1,1)=Llb(2,2)
         Llb(1,2)=-Llb(2,1)
      else  ! If a problem appear it returns 0...

         do i=1,4
            do j=1,4
               Llb(i,j)=0.d0
            enddo
         enddo
      endif
*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
* The inverse matrix is very simple after knowing the direct one

      do i=1,3
         do j=1,3
            Lbl(i,j)=Llb(j,i)  ! The transverse in the spatial components
         enddo
      enddo
      Lbl(4,4)=Llb(4,4)       ! One change of sign for each temporal component
      do i=1,3
         Lbl(4,i)=-Llb(i,4)
         Lbl(i,4)=-Llb(4,i)
      enddo

      end



