      subroutine eptogp(g,p,a,b)
c
C# --------------------------------------------------------
C#    subroutine eptogp(g,p,a,b)
C#    =================
C#
C@#  eptogp : transforms from e,p system to gamma,p system
C#
C#  This routine is used to transform from a 4-vector from
C#  the {\tt e, p} system to the {\tt gamma, p} system.
C#  The {\tt e, p} system is the laboratory system;
C#  the {\tt gamma, p} system is that
C#  system in which the photon and the proton collide
C#  along the {\tt z}-axis and in which the proton has the same
C#  momentum as in  {\tt e, p} system.
C# --------------------------------------------------------
*Start_Verbatim
C#   input :
C#
C#   g(4) ( real*4 ) - 4-momentum of photon in e,p system
C#   p(4) ( real*4 ) - 4-momentum of proton in e,p system
C#
C#   a(4) ( real*4 ) - 4-momentum in e,p system
C# --------------------------------------------------------
C#  output :
C#
C#   b(4) ( real*4 ) - 4-momentum in g,p system
C# --------------------------------------------------------
C#  author : Leonid K. Gladilin, Moscow State University
C#             email : gladilin@npi.msu.su
C#
C#    Date : 95-12-06
*********************************************************************
*End_Documentation

c
      dimension g(4),p(4),a(4),b(4)
c
      double precision
     &  dp(4),da(4),ds(4),dt(4),de(4),dd(4)
     & ,tp(4),sp(4)
     & ,rms,scs,ene,xcm,aa,e1,a1,a2,a3
c
      do i  = 1, 4
      dp(i) = p(i)
      ds(i) = g(i) + dp(i)
      da(i) = a(i)
      enddo
c
      if ( p(3) .lt. 0. )  then
      dp(3) = -dp(3)
      ds(3) = -ds(3)
      da(3) = -da(3)
                          endif
c
c   transformation to the  g,p  c.m.s
c
      rms = dsqrt ( ds(4)**2 - ds(3)**2
     &            - ds(2)**2 - ds(1)**2 )
c
      scs = ds(4)*dp(4) - ds(3)*dp(3)
     &    - ds(2)*dp(2) - ds(1)*dp(1)
c
      ene = scs/rms
c
      xcm =  - ( ene + dp(4) ) / ( rms + ds(4) )
c
      do i  = 1, 3
      tp(i) = dp(i) + ds(i)*xcm
      enddo
      tp(4) = ene
c
      scs = ds(4)*da(4) - ds(3)*da(3)
     &    - ds(2)*da(2) - ds(1)*da(1)
c
      ene = scs/rms
c
      xcm =  - ( ene + da(4) ) / ( rms + ds(4) )
c
      do i  = 1, 3
      dt(i) = da(i) + ds(i)*xcm
      enddo
      dt(4) = ene
c
c   rotation to the system with axis "z"
c   along proton
c
      aa = dsqrt( tp(1)**2 + tp(2)**2 + tp(3)**2 )
c
      do i = 1, 3
      de(i) = tp(i)/aa
      enddo
c
      e1 = de(3) + 1.d0
c
      a1 = de(3) + de(1)**2 / e1
      a2 = de(3) + de(2)**2 / e1
      a3 = de(1) * de(2)    / e1
c
c
      dd(1) =  dt(1)*a2    - dt(2)*a3    - dt(3)*de(1)
      dd(2) = -dt(1)*a3    + dt(2)*a1    - dt(3)*de(2)
      dd(3) =  dt(1)*de(1) + dt(2)*de(2) + dt(3)*de(3)
      dd(4) =  dt(4)
c
      sp(1) =  tp(1)*a2    - tp(2)*a3    - tp(3)*de(1)
      sp(2) = -tp(1)*a3    + tp(2)*a1    - tp(3)*de(2)
      sp(3) =  tp(1)*de(1) + tp(2)*de(2) + tp(3)*de(3)
      sp(4) =  tp(4)
c
c   transformation to the proton rest system
c
      rms = dsqrt ( dp(4)**2 - dp(3)**2
     &            - dp(2)**2 - dp(1)**2 )
c
      scs = sp(4)*dd(4) - sp(3)*dd(3)
     &    - sp(2)*dd(2) - sp(1)*dd(1)
c
      ene = scs/rms
c
      xcm =  - ( ene + dd(4) ) / ( rms + sp(4) )
c
      do i  = 1, 3
      dt(i) = dd(i) + sp(i)*xcm
      enddo
      dt(4) = ene
c
c   transformation to the system with initial
c   proton momentum
c
      scs = dt(4)*dp(4) + dt(3)*dp(3)
     &    + dt(2)*dp(2) + dt(1)*dp(1)
c
      ene = scs/rms
c
      xcm =  ( ene + dt(4) ) / ( rms + dp(4) )
c
      do i  = 1, 3
      b(i) = dt(i) + dp(i)*xcm
      enddo
      b(4) = ene
c
      if ( p(3) .lt. 0. )  b(3) = -b(3)
c
      return
      end


