//   nlo includes
#include "event.h"



namespace nlo {


  void lab_to_breit(event_dis& p)
  {
    lorentzvector<double> q(p[-1]-p[-2]);
    double xb = -0.5*q.mag2()/(p[0]*q);
    threevector<double> bVec = -((q + 2.0*xb*p[0]).boostVector());
    int i, low = p.lower(), up = p.upper();
    
    p[hadron(0)].boost(bVec);
    for(i = low; i <= up; i++) 
      p[i].boost(bVec);
    
    double phi = p[0].phi(), theta = p[0].theta();
    
    p[hadron(0)].rotateZ(-phi);
    p[hadron(0)].rotateY(-theta);  
    
    for(i = low; i <= up; i++) {
      p[i].rotateZ(-phi);
      p[i].rotateY(-theta);
    }
  }
  
  void breit_to_lab(double x, event_dis& p)
  {
    int i, low = p.lower(), up = p.upper();
    double bz = (1.0 - x)/(1.0 + x);
    threevector<double> bVec = -((p[-1] + p[hadron(0)]).boostVector());
    
    p[hadron(0)].boost(bVec);
    double phi = p[hadron(0)].phi(), theta = p[hadron(0)].theta();
    
    p[hadron(0)].rotateZ(-phi);
    p[hadron(0)].rotateY(-theta);  
    p[hadron(0)].boost(0.0, 0.0, bz);
    
    for(i = low; i <= up; i++) {
      p[i].boost(bVec);
      p[i].rotateZ(-phi);
      p[i].rotateY(-theta);
      p[i].boost(0.0, 0.0, bz);
    }
  }
}  // namespace nlo
