      Subroutine HCMINI(pbeam,pgam,Ierr)
C#*****************************************************************
C#      Subroutine HCMINI(pbeam,pgam,Ierr)
C#      Entry HCM(p,pnew,Ierr)
C#
C@# HCMINI/HCM (entry point) : Lorentz boost to hadronic centre of mass
C# PURPOSE : Lorentz boost to hadronic centre of mass
C#
C# INPUT :
C#   PBEAM    =  the 4 vector of the proton beam
C#   PGAM     =  the 4 vector of the virtual exchanged boson
C#   P        =   the vector to be acted upon
C# OUTPUT :
C#   PNEW     =  the boosted vector
C#   IERR     =  an error flag (1 = failed)
C#
C# HCMINI - called once an event to set up boost and rotation
C#          variables
C# HCM    - perform the actual boost of particles
C#
C# Author: N Brook
C#
C#*****************************************************************
*
      Real pbeam(4), pgam(4)
      Real p(4), phcm(4), pnew(4)
      Real Robo(3)
      Logical Lboost
*
      Ierr = 1
* set up boost parameters
      do 5 ivec = 1, 3
 5    robo(ivec) = (pbeam(ivec)+pgam(ivec))/(pbeam(4)+pgam(4))
*
      chksq = vdot(robo,robo,3)
      If(chksq.gt.1.) Return
* ... and necessary rotation angles
      call boost(-robo(1),-robo(2),-robo(3),pgam,phcm)
      theta = PHMANG(phcm(3),sqrt(phcm(2)**2+phcm(1)**2))
      phi   = PHMANG(phcm(1),phcm(2))
*
      Ierr = 0
      Return
*
      Entry HCM(p,pnew,Ierr)
*
      Ierr = 1
* Boost and rotate particle to HCM
      call boost(-robo(1),-robo(2),-robo(3),p,phcm)
      call phmrot(-phi,0.,phcm,phcm)
      call phmrot(0.,-theta,phcm,phcm)
      call ucopy(phcm,pnew,4)
*
      Ierr = 0
      Return
      End

* ============================================================


