      SUBROUTINE PUROBO(THE,PHI,BEX,BEY,BEZ,IERR)

C#==================================================================
C#      SUBROUTINE PUROBO(THE,PHI,BEX,BEY,BEZ,IERR)
C#
C@# PUROBO: perform rotations and boosts on PUPARTS "event record" --> PUTHRU.
C#   Purpose:
C#   To perform rotations and boosts on the PUPARTS "event record".
C#
*Start_Verbatim
C#   Input:
C#   The PUPARTS common block.  See the PUPARTS KEEP sequence
C#   documentation.
C#   THE,PHI: the standard polar coordinates theta and phi of
C#            the new frame, with respect to the old.
C#   BEX,BEY,BEZ: the speed, in units of c, of the frame to be
C#                boosted to, relative to the starting frame.
C#
C#   Output:
C#   The transformed momentum coordinates are returned in the
C#   PUPARTS common block, in place of the original ones.
C#   ierr = 0 : no error
C#          1 : requested subrange of PUPARTS is too big
C#
C#   Environment:
C#   PUROBO uses PHMROT and BOOST from the Phantom library.
C#
C#   Modification History:
C#   Modified from LUROBO of JETSET7.3 for use in PUTHRU on
C#   Nov 26, 1993 by Laurel Sinclair (McGill U.).
C#**********************************************************************
*End_Documentation


      IMPLICIT NONE

C...Global variables:
#include "puparts.inc"
      REAL THE,PHI,BEX,BEY,BEZ
      INTEGER IERR

C...Local variables:
      INTEGER IMIN,IMAX,IMI,IMA,I,J
      REAL PIN(3),POUT3(3),PIN4(4),POUT(4)

C...Find range of rotation/boost.
      IMIN=1
      IMAX=N
      GOTO 110

C...Entry for specific range.
      ENTRY PUDBRB(IMI,IMA,THE,PHI,BEX,BEY,BEZ,IERR)
      IMIN=IMI
      IF(IMIN.LE.0) IMIN=1
      IMAX=IMA
      IF(IMAX.LE.0) IMAX=N

 110  IERR = 0
C...Check rangeof rotation/boost.
      IF(IMIN.GT.NPARTMAX.OR.IMAX.GT.NPARTMAX) THEN
         WRITE(6,*) 'PUROBO: range outside PUPARTS memory'
         IERR = 1
         RETURN
      ENDIF

      DO I=IMIN,IMAX
         DO J=1,3
            PIN(J)=P(I,J)
         ENDDO
         CALL PHMROT(PHI,THE,PIN,POUT3)
         DO J=1,3
            PIN4(J)=POUT3(J)
         ENDDO
         PIN4(4) = P(I,4)
         CALL BOOST(BEX,BEY,BEZ,PIN4,POUT)
         DO J=1,4
            P(I,J)=POUT(J)
         ENDDO
      ENDDO

      RETURN
      END
* ============================================================



