      Subroutine BRTINI(pbeam,pgam,Ierr)
C#*****************************************************************
C#      Subroutine BRTINI(pbeam,pgam,Ierr)
C#      Entry BRT(p,pnew,Ierr)
C#
C@# BRTINI/BRT (entry point) : Lorentz boost to Breit frame
C# PURPOSE : Lorentz boost to Breit frame
C#
C# INPUT :
C#   PBEAM    =  the 4 vector of the proton beam
C#   PGAM     =  the 4 vector of the virtual exchanged boson
C#   P        =   the vector to be acted upon
C# OUTPUT :
C#   PNEW     =  the boosted vector
C#   IERR     =  an error flag (1 = failed)
C#
C# BRTINI - called once an event to set up boost and rotation
C#          variables
C# BRT    - perform the actual boost of particles
C#
C# Author: N Brook
C#
C#*****************************************************************
*
      Real pbeam(4), pgam(4)
      Real p(4), phcm(4), pnew(4)
      Real Robo(3)
      Logical Lboost
      Parameter(pi=3.141592654)
*
      Ierr = 1
      gammas = pgam(4)**2 - pgam(3)**2 - pgam(2)**2 - pgam(1)**2
      if(gammas.ge.0.0) Return
      xb = -gammas/2./(pgam(4)*pbeam(4)-pgam(3)*pbeam(3))
      if(xb.gt.1.0.or.xb.lt.0.0) Return
* set up boost parameters
      do 5 ivec = 1, 3
 5    robo(ivec) = (pgam(ivec)+2.*xb*pbeam(ivec))/
     &             (pgam(4)+2.*xb*pbeam(4))
*
*     chksq = vdot(robo,robo,3)
*     If(chksq.gt.1.) Return
* ... and necessary rotation angles
      call boost(-robo(1),-robo(2),-robo(3),pgam,phcm)
      theta = atan2(phcm(3),sqrt(phcm(2)**2+phcm(1)**2))
      phi   = atan2(phcm(1),phcm(2))
*
      Ierr = 0
      Return
*
      Entry BRT(p,pnew,Ierr)
*
      Ierr = 1
* Boost and rotate particle to Breit frame
      call boost(-robo(1),-robo(2),-robo(3),p,phcm)
      call phmrot(-phi,0.,phcm,phcm)
      call phmrot(0.,pi-theta,phcm,phcm)
      call ucopy(phcm,pnew,4)
*
      Ierr = 0
      Return
      End
* ============================================================



